//-- Main unit -----------------------------------------------------------------
//
//  ADllExports
//
//  Copyright  2001-2003 Alexander Kamburov
//  wise_guybg@yahoo.com
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
unit MainU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DllTools, StdCtrls, ComCtrls, ImgList, ToolWin, XPMan,
  Menus, ClipBrd, ActnList, ExtCtrls;

type
  TfrmMain = class(TForm)
    OpenDialog: TOpenDialog;
    ImageList: TImageList;
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ListView: TListView;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    ToolButton10: TToolButton;
    SaveDialog: TSaveDialog;
    XPManifest: TXPManifest;
    PopupMenu: TPopupMenu;
    Copy: TMenuItem;
    StatusBar: TStatusBar;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ActionList: TActionList;
    aGetFunctions: TAction;
    aFind: TAction;
    aNext: TAction;
    aSave: TAction;
    aRegister: TAction;
    aAbout: TAction;
    aExit: TAction;
    ListTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure CopyClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
    procedure ListViewColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListViewCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure aGetFunctionsExecute(Sender: TObject);
    procedure aFindExecute(Sender: TObject);
    procedure aNextExecute(Sender: TObject);
    procedure aSaveExecute(Sender: TObject);
    procedure aRegisterExecute(Sender: TObject);
    procedure aAboutExecute(Sender: TObject);
    procedure aExitExecute(Sender: TObject);
    procedure ListTimerTimer(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    FFirst: Boolean;
    procedure InitList(const AFileName: string);
    procedure FindText(const Indent: Integer = 0);
    function ListExport(const Name: String; Ordinal: Integer; Address: Pointer): Boolean;
  end;

var
  frmMain: TfrmMain;
  strToFind: string;
  ColumnToSort: Integer;

const
  strAppName = 'ADllExports';
  strAppVer  = 'v1.3';
  strAbout   = strAppName + ' ' + strAppVer + #13#10#13#10 +
               'Programmer: WISE GUY'#13#10 +
               'Country: BULGARIA'#13#10 +
               'E-mail: wise_guybg@yahoo.com'#13#10#13#10 +
               'Uses DLLTools unit from P. Below'#13#10#13#10 +
               'This software is released under the terms of'#13#10 +
               'the GNU General Public Licence v2';
  strStatus  = '%d export(s) found.';
  strLoading = 'Loading DLL exports...';

implementation

uses
  Math, Registry;

{$R *.DFM}

function TfrmMain.ListExport(const Name: String; Ordinal: Integer; Address: Pointer): Boolean;
var
  listentry: TListItem;
begin
  Result := true;
  listentry := ListView.Items.Add;
  listentry.Caption := Format('%p', [Address]);
  listentry.Subitems.Add(Format('%d', [Ordinal]));
  listentry.Subitems.Add(Name);

  //restart timer
  ListTimer.Enabled := false;
  ListTimer.Enabled := true;
end;

procedure TfrmMain.FormCreate(Sender: TObject);

  function GetSysDir: string;
  var s: string;
  begin
    SetLength(s, 256);
    GetSystemDirectory(PChar(s), 256);
    SetLength(s,StrLen(PChar(s)));
    Result := s;
  end;

begin
  Application.Title := strAppName;
  Caption := strAppName;
  OpenDialog.InitialDir := GetSysDir;
  FFirst := true;
end;

procedure TfrmMain.FindText(const Indent: Integer);
var
  i: Integer;
begin
  if strToFind = '' then
    Exit;

  for i := Indent to ListView.Items.Count - 1  do
  begin
    if Pos(strToFind, AnsiLowerCase(ListView.Items[i].SubItems[1])) > 0 then
    begin
      ListView.Scroll(ListView.Items[i].Left, ListView.Items[i].Top - 18);
      ListView.Selected := ListView.Items[i];
      Exit;
    end;
  end;

  ShowMessage('The text was not found.');
end;

procedure TfrmMain.CopyClick(Sender: TObject);
begin
  if not Assigned(ListView.Selected) then Exit;
  
  if ListView.Selected.SubItems.Count = 2 then
  begin
      Clipboard.AsText := Format('%s,%s,%s',
      [ListView.Selected.Caption,
       ListView.Selected.SubItems[0],
       ListView.Selected.SubItems[1]]);
  end
    else
      ShowMessage(Format('Error with subitems on line %d', [ListView.Selected.Index + 1]));
end;

procedure TfrmMain.PopupMenuPopup(Sender: TObject);
begin
  Copy.Enabled := Assigned(ListView.Selected);
end;

procedure TfrmMain.ListViewColumnClick(Sender: TObject; Column: TListColumn);
begin
  ColumnToSort := Column.Index;
  (Sender as TCustomListView).AlphaSort;
end;

procedure TfrmMain.ListViewCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  case ColumnToSort of
    0: Compare := CompareText(Item1.Caption, Item2.Caption);
    1: Compare := CompareValue(StrToInt(Item1.SubItems[0]), StrToInt(Item2.SubItems[0]));
    2: Compare := CompareText(Item1.SubItems[1], Item2.SubItems[1]);
  end;
end;

procedure TfrmMain.InitList(const AFileName: string);
begin
  try
    Caption := Format('%s - %s', [strAppName, AFileName]);
    StatusBar.SimpleText := strLoading;
    Application.ProcessMessages;
    ListView.Items.BeginUpdate;
    ListView.Items.Clear;
    ListDLLExports(AFileName, ListExport);
    ListTimer.Enabled := false;
    ListTimer.Enabled := true;
    OpenDialog.InitialDir := ExtractFilePath(AFileName);
  except
    ListTimerTimer(Self);
    Application.ProcessMessages;
    raise;
  end;
end;

procedure TfrmMain.aGetFunctionsExecute(Sender: TObject);
begin
  Application.ProcessMessages;  
  if OpenDialog.Execute then
    InitList(OpenDialog.FileName);
end;

procedure TfrmMain.aFindExecute(Sender: TObject);
begin
  if ListView.Items.Count = 0 then Exit;
  strToFind := AnsiLowerCase(InputBox(strAppName, 'Find text in Name :', ''));
  FindText;
end;

procedure TfrmMain.aNextExecute(Sender: TObject);
begin
  if Assigned(ListView.Selected) then
    FindText(ListView.Selected.Index + 1);
end;

procedure TfrmMain.aSaveExecute(Sender: TObject);
const
  SSaveComma = '%s,%s,%s'#13#10;
  SSaveTab   = '%s'#9'%s'#9'%s'#13#10;

  function WriteToFile: Boolean;
  var
    F: TFileStream;
    temp, formatstr: string;
    i: Integer;
  begin
    F := TFileStream.Create(SaveDialog.FileName, fmCreate);
    try
      if ExtractFileExt(SaveDialog.FileName) = '.csv' then
        formatstr := SSaveComma
        else
          formatstr := SSaveTab;
                    
      if ListView.Columns.Count = 3 then
      begin
        temp := Format(formatstr,
          [ListView.Column[0].Caption,
           ListView.Column[1].Caption,
           ListView.Column[2].Caption]);
        F.Write(temp[1], Length(temp));
      end
        else
          ShowMessage('Columns are more that 3. Please contact me...');

      for i := 0 to ListView.Items.Count - 1 do
        if ListView.Items[i].SubItems.Count = 2 then
        begin
          temp := Format(formatstr,
            [ListView.Items[i].Caption,
             ListView.Items[i].SubItems[0],
             ListView.Items[i].SubItems[1]]);
          F.Write(temp[1], Length(temp));
        end
          else
            ShowMessage(Format('Error with subitems on line %d', [i + 1]));
      Result := true;
    finally
      F.Free;
    end;
  end;

begin
  Application.ProcessMessages;

  if ListView.Items.Count = 0 then Exit;

  while SaveDialog.Execute do
  begin
    if FileExists(SaveDialog.FileName) then
    begin
      case MessageDlg('File exists. Overwrite?', mtConfirmation, mbYesNoCancel, 0) of
        ID_CANCEL: Exit;
        ID_YES   : if WriteToFile then Exit;
      end;
    end
      else
        if WriteToFile then Exit;      
  end;
end;

procedure TfrmMain.aRegisterExecute(Sender: TObject);
var
  Reg: TRegistry;
  DllHome: string;
const
  dll_ext = '.dll';
  dll_shell = 'shell';
  dll_withade = 'openwithadllexp';
  dll_withade_str = 'Open with ADllExports';
  dll_cmd = 'command';
  dll_cmd_str = '"%s" "%%1"';
  dll_reg   = 'Register "Open with ADllExports" command in Explorer''s context menu?';
  dll_unreg = 'Unregister "Open with ADllExports" command in Explorer''s context menu?';
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey(dll_ext, false) then
    begin
      DllHome := Reg.ReadString('');

      if Length(DllHome) > 0 then
      begin
        Reg.CloseKey;
        if Reg.OpenKey(DllHome, false) then
          if Reg.OpenKey(dll_shell, false) then
            if Reg.KeyExists(dll_withade) then
            begin
              if MessageDlg(dll_unreg, mtConfirmation, mbOKCancel, 0) = IDOK then
                Reg.DeleteKey(dll_withade);
            end
              else
                if MessageDlg(dll_reg, mtConfirmation, mbOKCancel, 0) = IDOK then
                begin
                  Reg.OpenKey(dll_withade, true);
                  Reg.WriteString('', dll_withade_str);
                  Reg.OpenKey(dll_cmd, true);
                  Reg.WriteString('', Format(dll_cmd_str, [Application.ExeName]));
                end;
      end;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TfrmMain.aAboutExecute(Sender: TObject);
begin
  MessageDlg(strAbout, mtInformation, [mbOK], 0);
end;

procedure TfrmMain.aExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.ListTimerTimer(Sender: TObject);
begin
  ListTimer.Enabled := false;
  ListView.Items.EndUpdate;
  StatusBar.SimpleText := Format(strStatus, [ListView.Items.Count]);
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  if FFirst then
  begin
    FFirst := false;
    if ParamCount > 0 then
      InitList(ParamStr(1));
  end;
end;

end.
